import datetime
from pathlib import Path

import pytest

from um_ro_calendar.entities import CheckboxOption, FormFeatures, TermOption
from um_ro_calendar.fetching import FileFetcher
from um_ro_calendar.html_source import HTMLEventParser, HTMLFetcher, parse_html_dates

example_features = FormFeatures(
    "term_keyword",
    [TermOption("1", "Fall 2022"), TermOption("2", "Winter 2023")],
    CheckboxOption("accal", "1"),
    CheckboxOption("reg", "1"),
)


def test_html_fetcher():
    fetcher = HTMLFetcher(example_features)
    url = fetcher.get_url("Winter 2023")
    assert "calendars" in url
    assert "feed" not in url
    assert "term_keyword=2" in url


def test_successful_parse():
    data_file = Path(__file__).parent / "data" / "sample-winter2023-accalreg.html"
    fetcher = FileFetcher(data_file)
    parser = HTMLEventParser(fetcher)
    events = list(parser.events("Winter 2023"))
    assert len(events) == 36
    assert events[0].start_date.year == 2022
    assert events[2].start_date == datetime.date(2022, 11, 17)
    assert events[2].end_date == datetime.date(2022, 12, 5)
    assert events[-3].end_date == datetime.date(2023, 4, 21)
    assert events[-1].start_date == datetime.date(2023, 4, 27)
    assert events[-1].end_date == datetime.date(2023, 4, 30)


def test_parses_around_html():
    data_file = Path(__file__).parent / "data" / "sample-winter2023-accalreg.html"
    fetcher = FileFetcher(data_file)
    parser = HTMLEventParser(fetcher)
    events = list(parser.events("Winter 2023"))
    assert events[21].name.startswith("No reduction in tuition")


@pytest.mark.parametrize(
    "date_string,wrap_after,expected",
    [
        ("Feb 5, Mon", 13, [(datetime.date(2023, 2, 5), datetime.date(2023, 2, 5))]),
        (
            "Feb 5-6, Mon-Tues",
            13,
            [(datetime.date(2023, 2, 5), datetime.date(2023, 2, 6))],
        ),
        (
            "June 5, Fri & June 8-10, Mon-Wed",
            13,
            [
                (datetime.date(2023, 6, 5), datetime.date(2023, 6, 5)),
                (datetime.date(2023, 6, 8), datetime.date(2023, 6, 10)),
            ],
        ),
        (
            "Dec 6, Thurs",
            13,
            [(datetime.date(2023, 12, 6), datetime.date(2023, 12, 6))],
        ),
        ("Dec 6, Thurs", 8, [(datetime.date(2022, 12, 6), datetime.date(2022, 12, 6))]),
        (
            "Nov 17–Dec 5, Thur-Mon",
            13,
            [(datetime.date(2023, 11, 17), datetime.date(2023, 12, 5))],
        ),
        (
            "Apr 20-21, Thurs-Fri & Apr 24-27, Mon-Thurs",
            13,
            [
                (datetime.date(2023, 4, 20), datetime.date(2023, 4, 21)),
                (datetime.date(2023, 4, 24), datetime.date(2023, 4, 27)),
            ],
        ),
    ],
)
def test_date_parsing(date_string, wrap_after, expected):
    base_year = 2023
    assert list(parse_html_dates(date_string, base_year, wrap_after)) == expected
